#!/bin/bash
# Script to create a new RW_LEGACY section of a Chromebook rom

CBFSTOOL=cbfstool
ORIGIMAGE=origimage.rom
PAYLOAD=out/bios.bin.elf
PAYLOADVGA=out/vgabios.bin
FINALIMAGE=newimage.rom

# Make sure cbfstool is available
which $CBFSTOOL > /dev/null
if [ $? != 0 ]; then
    echo "ERROR - could not find $CBFSTOOL"
    echo "Please add cbfstool to the path"
    exit 1
fi

# Make sure original rom was extracted and copied to this machine.
if [[ ! -f $ORIGIMAGE ]]; then
    echo "ERROR - Please extract the original rom image from target machine."
    echo "Run something like the following on the target machine:"
    echo "    sudo flashrom -r /tmp/origimage.rom"
    echo "    scp /tmp/origimage.rom thismachine:/path/to/here/"
    exit 1
fi

# Build SeaBIOS with CONFIG_COREBOOT=y, CONFIG_CBFS_LOCATION=0xffe00000,
# and CONFIG_VGA_COREBOOT=y and place final bios.bin.elf at $PAYLOAD
if [[ ! -f $PAYLOAD || ! -f $PAYLOADVGA ]]; then
    echo "ERROR - Please build SeaBIOS (with at least CONFIG_COREBOOT=y, "
    echo "    CONFIG_CBFS_LOCATION=0xffe00000, CONFIG_VGA_COREBOOT=y)"
    exit 1
fi

# Build new seabios CBFS image.
CBFSFILE=seabios.cbfs
dd if=/dev/zero of=bootblock count=1 bs=64 2> /dev/null
$CBFSTOOL $CBFSFILE create -s $(( 2*1024*1024 )) -B bootblock -m x86

# Add SeaBIOS payload to CBFS
$CBFSTOOL $CBFSFILE add-payload -f $PAYLOAD -n payload -c lzma

## Extract vgabios from SeaBIOS CBFS area of original image.
#ORIGCBFSFILE=orig-seabios.cbfs
#dd if=$ORIGIMAGE of=$ORIGCBFSFILE skip=2 bs=2M count=1 2> /dev/null
#$CBFSTOOL $ORIGCBFSFILE extract -n pci8086,0406.rom -f pci8086,0406.rom
#$CBFSTOOL $CBFSFILE add -f pci8086,0406.rom -n pci8086,0a06.rom -t optionrom

# Add SeaVGABIOS to CBFS
$CBFSTOOL $CBFSFILE add -f $PAYLOADVGA -n vgaroms/seavgabios.rom -t optionrom

## Other settings that one could set
#$CBFSTOOL $CBFSFILE add -f bootorder -n bootorder -t raw
#$CBFSTOOL $CBFSFILE add-int -i 3000 -n etc/ps2-keyboard-spinup
#$CBFSTOOL $CBFSFILE add-int -i 1 -n etc/ps2-poll-only

# Display the final contents of the CBFS file
$CBFSTOOL $CBFSFILE print

# Combine seabios CBFS with original rom to build final image.
cp $ORIGIMAGE $FINALIMAGE
dd if=$CBFSFILE of=$FINALIMAGE seek=2 bs=2M conv=notrunc 2> /dev/null

# Final image write instructions
echo ""
echo ""
echo "Produced final image file: $FINALIMAGE"
echo "Use it by running something like the following on the target machine:"
echo "    scp thismachine:/path/to/here/newimage.rom /tmp/"
echo "    sudo flashrom -w /tmp/newimage.rom -i RW_LEGACY"
